--
-- Iron Man
-- Version: IMR 1.0.100205
-- Author: Walkerbo
-- Email: walkerbo.saurfang@gmail.com
-- Date: Wednesday 17 January 2024
--

-- I have tried to name variables, functions, frames, and buttons to be descriptive

-- ===================================================================================================
-- ===================================================================================================
-- ===========================         Metrics, Lists & Variables          ===========================
-- ===================================================================================================
-- ===================================================================================================

local addonName = "Iron Man"
local versionNumber = "IMR 1.0.100205"

-- variables for the get all details, travel and build string functions
local DateTable,
    TemporaryTable,
    TimeString,
    TimeDateString,
    NameClassRaceString,
    NewLevel,
    TravelSpellID,
    CharClass,
    CharRace,
    CharSex,
    CharName = {}, {}, "", "", "", 0, 0, "", "", "", ""

local ImageList = {
    blue = "Interface\\AddOns\\IronMan\\Files\\Images\\IMBlue",
    goldRed = "Interface\\AddOns\\IronMan\\Files\\Images\\IMRedGold",
    green = "Interface\\AddOns\\IronMan\\Files\\Images\\IMGreen",
    grey = "Interface\\AddOns\\IronMan\\Files\\Images\\IMGrey",
    IMBanner = "Interface\\AddOns\\IronMan\\Files\\Images\\IronManBanner",
    IMBG = "Interface\\AddOns\\IronMan\\Files\\Images\\IMBG5",
    orange = "Interface\\AddOns\\IronMan\\Files\\Images\\IMOrange",
    pink = "Interface\\AddOns\\IronMan\\Files\\Images\\IMPink",
    purple = "Interface\\AddOns\\IronMan\\Files\\Images\\IMPurple"
}

local NumberList = {
    achievementCloseButtonX = -5,
    achievementCloseButtonY = -6,
    achievementFrameHeight = 160,
    achievementFramePositionY = 175,
    achievementFrameWidth = 600,
    bottomLeftTextureX = -200,
    bottomRightTextureX = 200,
    buttonSpacerBottom = -20,
    buttonSpacerLeft = 200,
    buttonSpacerTop = 30,
    dateNameTextBoxHeight = 40,
    warningDetailsTextBoxHeight = 80,
    warningDetailsTextBoxWidth = 380,
    dateNameTextBoxWidth = 500,
    dateTextBoxBottomGap = 5,
    edgeSize = 20,
    failTextBoxHeight = 80,
    failTextBoxSpacerLeft = -20,
    failTextBoxSpacerTop = -20,
    failTextBoxWidth = 380,
    iconAchievementTextureScale = .25,
    iconTextureScale = .45,
    iconTextureX = -11,
    iconTextureY = 30,
    insetSize = 5,
    interfaceTextureBoxSize = 50,
    interfaceTextureGap = -50,
    interfaceTextureScale = .55,
    maxDeaths = 100,
    maxFails = 100,
    maxJumps = 100000, -- debug -- change for testing
    maxLevel = GetMaxLevelForPlayerExpansion(), -- debug -- change for testing
    maxPlayed = 100,
    metaAchievementFramePositionGapY = -40,
    metaAchievementTopGap = -8,
    nameTextBoxBottomGap = -5,
    rulessTextBoxLeftWidth = 210,
    rulessTextBoxRightWidth = 400,
    rulesTextBoxLeftHeight = 420,
    rulesTextBoxLeftSideGap = -145,
    rulesTextBoxLeftTopGap = 30,
    rulesTextBoxRightHeight = 420,
    scrollAchievementTextBoxHeight = 40,
    scrollAchievementTextBoxWidth = 350,
    scrollAchievementTextureScale = .25,
    scrollFrameNumberOfFrames = 5,
    scrollFrameRowHeight = 80,
    scrollFrameRowWidth = 400,
    scrollFrameWidth = 330,
    scrollParentGapX = 10,
    scrollParentHeight = 410,
    scrollParentTopGap = -135,
    scrollParentWidth = 435,
    selectionButtonHeight = 45,
    selectionButtonWidth = 190,
    selectionFrameHeight = 200,
    selectionFramePositionY = 175,
    selectionFrameScale = 1.2,
    selectionFrameWidth = 500,
    textBoxHeight = 100,
    textBoxSpacerLeft = 0,
    textBoxSpacerTop = 0,
    textBoxWidth = 190,
    titleTextBoxHeight = 80,
    titleTextBoxTopGap = -10,
    titleTextBoxWidth = 350
}

local FontList = {
    failFont = "Fancy32Font",
    nameDateFont = "Fancy22Font",
    rulesFont = "NumberFontNormalLargeYellow",
    titleFont = "Fancy48Font",
    failColour = "|cffff0000", -- red
    loadColour = "|cffffb90f", -- gold
    titleColour = "|cff00ff00" -- green
}

local StringList = {
    ironManLoad = (FontList.loadColour .. " *** " .. addonName .. " - " .. versionNumber .. " Loaded *** \n"),
    acceptChallenge = "Accept Ironman Challenge",
    continueChallenge = "Continue Ironman Challenge",
    closeFrameAfterFail = "Goodbye",
    doNotAcceptChallenge = "No Thanks",
    doNotContinueChallenge = "Cancel Ironman Challenge",
    rulesButton = "Rules",
    ironManWarning = (FontList.failColour .. "Warning"),
    ironManWarningTravel = (FontList.failColour .. "You are using a travel spell.\nCancel this spell or you will fail"),
    ironManWarningInstance = (FontList.failColour .. "You have entered an instance.\nExit the instance or you will fail"),
    ironManWarningPetBattle = (FontList.failColour ..
        "You have started a pet battle.\nForfiet the pet battle or you will fail"),
    ironManShape = (FontList.failColour ..
        "You have changed shape into a prohibited shape.\nChange back or you will fail"),
    ironManPetBattleFail = (FontList.failColour .. "You have participated in a pet battle"),
    ironManTravelFail = (FontList.failColour .. "You cast a travel spell"),
    ironManBoostedFail = (FontList.failColour .. "You boosted"),
    ironManDiedFail = (FontList.failColour .. "You died"),
    ironManEquippedFail = (FontList.failColour .. "You equipped an item that was not grey"),
    ironManGroupFail = (FontList.failColour .. "You joined a group"),
    ironManGuildFail = (FontList.failColour .. "You joined a guild"),
    ironManInstanceFail = (FontList.failColour .. "You entered an instance"),
    ironManShapeFail = (FontList.failColour .. "You used a prohibited shape"),
    ironManRulesShort = ("Level to max without dying following these rules."),
    ironManRules = ("1. Do not die.\n2. Do not boost.\n3. No proffessions.\n4. No gear other than starting weapons.\n5. No mounts.\n6. Do not join a party.\n7. Do not join a raid.\n8. Do not join a guild.\n9. Do not start a guild.\n10. No flight paths.\n11. No pet battles.\n12. No mail.\n13. No auction house.\n14. No hearthstones.\n15. No mage portals.\n16. No mage teleports.\n17. No instances.\n18. No trades.\n19. No heirlooms.\n20. No toys.\n21. No macros.\n22. No death gate.\n23. No Druid travel forms.\n24. No Shaman ghost wolf form.\n25. No mission tables."),
    ironManRulesDetails = ("This addon will help you adhere to the rules by disabling or blocking certain functions.\n\nThings that are blocked or disabled;\nProfession trainers. Equipping items. Access to your collections. Access to your macros. Joining a guild, party or raid. Access to the mail box and auction house. Accessing guild masters. Accessing your equipment manager. Mission tables. Trading with other players.\n\nThings that cannot be disabled that will cause a fail;\nDeath. Using hearthstones. Using shaman astral recall. Using shaman ghost wolf. Using druid travel forms. Using mage portals and teleports. Using a death knight death gate. Entering an instance.Boosting your character.\n\nOnly new characters can take on the Iron Man challenge.\n\nHitting max level without dying will display your achievement and take a screenshot."),
    ironManSuccess = (FontList.titleColour .. "I AM IRON MAN"),
    ironManDeaths = (FontList.failColour .. "1000 Deaths"),
    ironManFails = (FontList.failColour .. "1000 Fails"),
    ironManJumps = (FontList.titleColour .. "100,000 Jumps"), -- debug --
    ironManAllRaces = (FontList.titleColour .. "All Races"),
    ironManAllClasses = (FontList.titleColour .. "All Classes"),
    ironManAllGenders = (FontList.titleColour .. "All Genders")
}

--[[
local SoundList = { -- debug --
    failServices = "Interface\\AddOns\\IronMan\\Files\\Sounds\\services.wav",
    iAmIronMan = "Interface\\AddOns\\IronMan\\Files\\Sounds\\IAmIronMan.wav",
    nextTime = "Interface\\AddOns\\IronMan\\Files\\Sounds\\NextTime.wav",
    suitUp = "Interface\\AddOns\\IronMan\\Files\\Sounds\\IronManSuitUp.wav"
}
--]]

-- list of guild masters used to prevent starting a guild
-- some guild masters have multiple id's as they are found in different areas
local guildMasterVendorList = {
    3370, -- Urtrun Clanbringer
    4161, -- Lysheana
    4613, -- Christopher Drakul
    4974, -- Aldwin Laughlin
    5054, -- Krumn
    5130, -- Jondor Steelbrow
    16568, -- Tandrine
    16734, -- Funaam
    28774, -- Andrew Matthews
    52277, -- Droha
    96811, -- Andrew Matthews
    108092, -- Aldwin Laughlin
    130964, -- Christopher Drakul
    142077, -- Dockwarden Finnson
    142179, -- Cringely Rottenfist
    152324, -- Aldwin Laughlin
    177066 -- Lysheana
}

-- hearthstones, teleports, portals, items, toys
local travelList = {
    556, -- astral recall
    3561, -- teleport-stormwind
    3562, -- teleport-ironforge
    3563, -- teleport-undercity
    3565, -- teleport-darnassus
    3566, -- teleport-thunder-bluff
    3567, -- teleport-orgrimmar
    8690, -- hearthstone
    10059, -- portal-stormwind
    11416, -- portal-ironforge
    11417, -- portal-orgrimmar
    11418, -- portal-undercity
    11419, -- portal-darnassus
    11420, -- portal-thunder-bluff
    18960, -- teleport-moonglade
    32266, -- portal-exodar
    32267, -- portal-silvermoon
    32271, -- teleport-exodar
    32272, -- teleport-silvermoon
    33690, -- teleport-shattrath
    33691, -- portal-shattrath
    35715, -- teleport-shattrath
    35717, -- portal-shattrath
    41234, -- black temple
    49358, -- teleport-stonard
    49359, -- teleport-theramore
    49360, -- portal-theramore
    49361, -- portal-stonard
    50977, -- death knight death gate
    53140, -- teleport-dalaran-northrend
    53142, -- portal-dalaran-northrend
    88342, -- teleport-tol-barad
    88344, -- teleport-tol-barad
    88345, -- portal-tol-barad
    88346, -- portal-tol-barad
    120145, -- ancient-teleport-dalaran
    120146, -- ancient-portal-dalaran
    132620, -- portal-vale-of-eternal-blossoms
    132621, -- teleport-vale-of-eternal-blossoms
    132626, -- portal-vale-of-eternal-blossoms
    132627, -- teleport-vale-of-eternal-blossoms
    171253, -- garrison hearthstone
    176242, -- teleport-warspear
    176244, -- portal-warspear
    176246, -- portal-stormshield
    176248, -- teleport-stormshield
    -- 193759, -- teleport-hall-of-the-guardian -- debug -- legion class hall for mages
    222695, -- legion, -- dalaran hearthstone
    224869, -- teleport-dalaran-broken-isles
    281400, -- portal-boralus
    281402, -- portal-dazaralor
    281403, -- teleport-boralus
    281404, -- teleport-dazaralor
    343127, -- teleport
    343140, -- portal
    344587, -- teleport-oribos
    344597, -- portal-oribos
    395277, -- teleport-valdrakken
    395289 -- portal-valdrakken
}

local InventorySlots = {
    -- Comment or remove whichever slots you don't want to process, these enums exist in all clients
    INVSLOT_AMMO,
    INVSLOT_HEAD,
    INVSLOT_NECK,
    INVSLOT_SHOULDER,
    INVSLOT_BODY,
    INVSLOT_CHEST,
    INVSLOT_WAIST,
    INVSLOT_LEGS,
    INVSLOT_FEET,
    INVSLOT_WRIST,
    INVSLOT_HAND,
    INVSLOT_FINGER1,
    INVSLOT_FINGER2,
    INVSLOT_TRINKET1,
    INVSLOT_TRINKET2,
    INVSLOT_BACK,
    --    INVSLOT_MAINHAND, -- need weapons
    --    INVSLOT_OFFHAND, -- need weapons
    --    INVSLOT_RANGED, -- need weapons
    INVSLOT_TABARD
}

-- ===================================================================================================
-- ===================================================================================================
-- ===========================         Functions          ===========================
-- ===================================================================================================
-- ===================================================================================================

local function initialiseTables()
    if not IronManGlobalVariables then
        IronManGlobalVariables = {
            metaClasses = false,
            metaClassesSexes = false,
            metaDeaths = false,
            metaFails = false,
            metaJumps = false,
            metaPlayed = false,
            metaRaces = false,
            metaRacesSexes = false,
            totalNumberOfDeaths = 0,
            totalNumberOfFails = 0,
            totalNumberOfJumps = 0,
            totalTimePlayed = 0
        }
        IronManCompleatedAchievements = {}
        IronManClassTable = {
            -- unknown/neutrum = 1  --  male = 2  --   female = 3
            {class = "Warrior", male = false, female = false, classID = 1, startingLevel = 1},
            {class = "Paladin", male = false, female = false, classID = 2, startingLevel = 1},
            {class = "Hunter", male = false, female = false, classID = 3, startingLevel = 1},
            {class = "Rogue", male = false, female = false, classID = 4, startingLevel = 1},
            {class = "Priest", male = false, female = false, classID = 5, startingLevel = 1},
            {class = "Deathknight", male = false, female = false, classID = 6, startingLevel = 10},
            {class = "Shaman", male = false, female = false, classID = 7, startingLevel = 1},
            {class = "Mage", male = false, female = false, classID = 8, startingLevel = 1},
            {class = "Warlock", male = false, female = false, classID = 9, startingLevel = 1},
            {class = "Monk", male = false, female = false, classID = 10, startingLevel = 1},
            {class = "Druid", male = false, female = false, classID = 11, startingLevel = 1},
            {class = "Demonhunter", male = false, female = false, classID = 12, startingLevel = 8},
            {class = "Evoker", male = false, female = false, classID = 13, startingLevel = 58}
        }
        IronManRaceTable = {
            {race = "Orc", male = false, female = false, startingLevel = 1},
            {race = "Undead", male = false, female = false, startingLevel = 1},
            {race = "Tauren", male = false, female = false, startingLevel = 1},
            {race = "Troll", male = false, female = false, startingLevel = 1},
            {race = "Blood Elf", male = false, female = false, startingLevel = 1},
            {race = "Goblin", male = false, female = false, startingLevel = 1},
            {race = "Pandaren", male = false, female = false, startingLevel = 1},
            {race = "Nightborne", male = false, female = false, startingLevel = 10},
            {race = "Highmountain Tauren", male = false, female = false, startingLevel = 10},
            {race = "Mag'har Orc", male = false, female = false, startingLevel = 10},
            {race = "Zandalari Troll", male = false, female = false, startingLevel = 10},
            {race = "Vulpera", male = false, female = false, startingLevel = 10},
            {race = "Human", male = false, female = false, startingLevel = 1},
            {race = "Dwarf", male = false, female = false, startingLevel = 1},
            {race = "Night Elf", male = false, female = false, startingLevel = 1},
            {race = "Gnome", male = false, female = false, startingLevel = 1},
            {race = "Draenei", male = false, female = false, startingLevel = 1},
            {race = "Worgen", male = false, female = false, startingLevel = 1},
            {race = "Void Elf", male = false, female = false, startingLevel = 10},
            {race = "Lightforged Draenei", male = false, female = false, startingLevel = 10},
            {race = "Dark Iron Dwarf", male = false, female = false, startingLevel = 10},
            {race = "Kul Tiran", male = false, female = false, startingLevel = 10},
            {race = "Mechagnome", male = false, female = false, startingLevel = 10},
            {race = "Dracthyr", male = false, female = false, startingLevel = 58}
        }
    end
    if not IronManCharacterVariables then
        IronManCharacterVariables = {
            isNew = true,
            isAttemptingIronMan = false,
            completedIronMan = false,
            failedIronMan = false,
            numberOfJumps = 0,
            timePlayed = 0,
            logoutLevel = 1,
            evokerDeathknightWeaponEquipped = false
        }
    end
end

local function unregisterEventsIfNotRequired() -- if not attempting then turn off events
    IronManSelectionFrame:Hide() -- debug --
    IronManSelectionFrame:UnregisterEvent("ADVENTURE_MAP_OPEN")
    IronManSelectionFrame:UnregisterEvent("AUCTION_HOUSE_SHOW")
    IronManSelectionFrame:UnregisterEvent("EQUIP_BIND_CONFIRM")
    IronManSelectionFrame:UnregisterEvent("GARRISON_ARCHITECT_OPENED")
    IronManSelectionFrame:UnregisterEvent("GARRISON_MISSION_NPC_OPENED")
    IronManSelectionFrame:UnregisterEvent("GARRISON_SHIPYARD_NPC_OPENED")
    IronManSelectionFrame:UnregisterEvent("GOSSIP_SHOW")
    IronManSelectionFrame:UnregisterEvent("GROUP_FORMED")
    IronManSelectionFrame:UnregisterEvent("GUILD_INVITE_REQUEST")
    IronManSelectionFrame:UnregisterEvent("MAIL_SHOW")
    IronManSelectionFrame:UnregisterEvent("MERCHANT_SHOW")
    IronManSelectionFrame:UnregisterEvent("PARTY_INVITE_REQUEST")
    IronManSelectionFrame:UnregisterEvent("PET_BATTLE_OPENING_DONE")
    IronManSelectionFrame:UnregisterEvent("PETITION_SHOW")
    IronManSelectionFrame:UnregisterEvent("PLAYER_DEAD")
    IronManSelectionFrame:UnregisterEvent("PLAYER_ENTERING_WORLD")
    IronManSelectionFrame:UnregisterEvent("PLAYER_EQUIPMENT_CHANGED")
    IronManSelectionFrame:UnregisterEvent("PLAYER_INTERACTION_MANAGER_FRAME_SHOW")
    IronManSelectionFrame:UnregisterEvent("PLAYER_LEVEL_CHANGED")
    IronManSelectionFrame:UnregisterEvent("PLAYER_LOGOUT")
    IronManSelectionFrame:UnregisterEvent("TAXIMAP_OPENED")
    IronManSelectionFrame:UnregisterEvent("TIME_PLAYED_MSG")
    IronManSelectionFrame:UnregisterEvent("TRADE_SHOW")
    IronManSelectionFrame:UnregisterEvent("TRAINER_SHOW")
    IronManSelectionFrame:UnregisterEvent("UNIT_SPELLCAST_SENT")
    IronManSelectionFrame:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED")
    IronManSelectionFrame:UnregisterEvent("UPDATE_SHAPESHIFT_FORM")
end

local function failedIronManChallenge(failType)
     -- print("fail fired", failType) -- debug --
    if IronManCharacterVariables.isAttemptingIronMan then
        if failType == "Died" then
            IronManSelectionFrameTextBoxFailText:SetText(StringList.ironManDiedFail)
            IronManGlobalVariables.totalNumberOfDeaths = IronManGlobalVariables.totalNumberOfDeaths + 1
        elseif failType == "Boosted" then
            IronManSelectionFrameTextBoxFailText:SetText(StringList.ironManBoostedFail)
        elseif failType == "EnteredInstance" then
            IronManSelectionFrameTextBoxFailText:SetText(StringList.ironManInstanceFail)
        elseif failType == "JoinedGroup" then
            IronManSelectionFrameTextBoxFailText:SetText(StringList.ironManGroupFail)
        elseif failType == "JoinedGuild" then
            IronManSelectionFrameTextBoxFailText:SetText(StringList.ironManGuildFail)
        elseif failType == "Equiped" then
            IronManSelectionFrameTextBoxFailText:SetText(StringList.ironManEquippedFail)
        elseif failType == "ShapeShifted" then
            IronManSelectionFrameTextBoxFailText:SetText(StringList.ironManShapeFail)
        elseif failType == "Travel" then
            IronManSelectionFrameTextBoxFailText:SetText(StringList.ironManTravelFail)
        elseif failType == "PetBattle" then
            IronManSelectionFrameTextBoxFailText:SetText(StringList.ironManPetBattleFail)
        end
    end

    IronManCharacterVariables.isAttemptingIronMan = false
    IronManCharacterVariables.completedIronMan = false
    IronManCharacterVariables.failedIronMan = true
    IronManGlobalVariables.totalNumberOfFails = IronManGlobalVariables.totalNumberOfFails + 1
    -- PlaySoundFile(SoundList.failServices) -- debug --

    IronManSelectionFrameiconTexture:SetTexture(ImageList.grey)
    unregisterEventsIfNotRequired()
    IronManSelectionFramecloseButton:SetText(StringList.closeFrameAfterFail)
    IronManSelectionFrameselectionButton:Hide()
    IronManSelectionFrameTextBoxText:Hide()
    IronManSelectionFrameTextBoxFail:Show()
    IronManSelectionFrameTextBoxFailText:Show()
    IronManSelectionFrame:Show()
end

local function closeVendorIfIsAGuildMaster() -- need to prevent access to guild masters
    local vendorGUID = UnitGUID("target")
    if not vendorGUID then
        return
    end
    local _, _, _, _, _, npcID, _ = strsplit("-", vendorGUID)
    local vendorID = tonumber(npcID)
    for k, v in pairs(guildMasterVendorList) do
        if v == vendorID then
            CloseMerchant()
            C_GossipInfo.CloseGossip()
        end
    end
end

local function updateAchievementList() -- update for completed achievements scroll frame
    FauxScrollFrame_Update(
        IronManInterfaceScrollFrame,
        #IronManCompleatedAchievements,
        NumberList.scrollFrameNumberOfFrames,
        NumberList.scrollFrameRowHeight
    )
    local os = FauxScrollFrame_GetOffset(IronManInterfaceScrollFrame)
    for index = 1, NumberList.scrollFrameNumberOfFrames do
        local frame = IronManInterfaceScrollFrame.frames[index]
        local offset = os + index
        if offset <= #IronManCompleatedAchievements then
            local row = IronManInterfaceScrollFrame.frames[index]
            row.texture:SetTexture(IronManCompleatedAchievements[offset].icon)
            row.titleTextBox.text:SetText(IronManCompleatedAchievements[offset].title)
            row.frameNameTextBox.text:SetText(IronManCompleatedAchievements[offset].char)
            row.frameDateTextBox.text:SetText(IronManCompleatedAchievements[offset].timeDate)
            row:Show()
        else
            row:Hide()
        end
    end
end

local function updateClassRaceTables() -- update tables when achievement reached
    for k, v in pairs(IronManClassTable) do
        if v.class == CharClass then
            if CharSex == "Male" then
                v.male = true
            end
            if CharSex == "Female" then
                v.female = true
            end
        end
    end

    for k, v in pairs(IronManRaceTable) do
        if v.race == CharRace then
            if CharSex == "Male" then
                v.male = true
            end
            if CharSex == "Female" then
                v.female = true
            end
        end
    end
end

local function buildStrings() -- build char date strings
    print("time played build string", TimeString)
    local days = math.floor(TimeString / (24 * 3600))
    local hours = math.floor((TimeString % (24 * 3600)) / 3600)
    local minutes = math.floor((TimeString % 3600) / 60)
    TimeString = string.format("%d days, %d hours, %d minutes", days, hours, minutes)

    local characterGUID = UnitGUID("player")
    local class, _, race, _, sex, name = GetPlayerInfoByGUID(characterGUID)
    local faction = UnitFactionGroup("Player")

    for k, v in pairs(date("*t")) do
        DateTable[k] = v
    end
    if DateTable.wday == 1 then
        DateTable.wday = "Sun"
    elseif DateTable.wday == 2 then
        DateTable.wday = "Mon"
    elseif DateTable.wday == 3 then
        DateTable.wday = "Tue"
    elseif DateTable.wday == 4 then
        DateTable.wday = "Wed"
    elseif DateTable.wday == 5 then
        DateTable.wday = "Thu"
    elseif DateTable.wday == 6 then
        DateTable.wday = "Fri"
    elseif DateTable.wday == 7 then
        DateTable.wday = "Sat"
    end

    if DateTable.month == 1 then
        DateTable.month = "Jan"
    elseif DateTable.month == 2 then
        DateTable.month = "Feb"
    elseif DateTable.month == 3 then
        DateTable.month = "Mar"
    elseif DateTable.month == 4 then
        DateTable.month = "Apr"
    elseif DateTable.month == 5 then
        DateTable.month = "May"
    elseif DateTable.month == 6 then
        DateTable.month = "Jun"
    elseif DateTable.month == 7 then
        DateTable.month = "Jul"
    elseif DateTable.month == 8 then
        DateTable.month = "Aug"
    elseif DateTable.month == 9 then
        DateTable.month = "Sep"
    elseif DateTable.month == 10 then
        DateTable.month = "Oct"
    elseif DateTable.month == 11 then
        DateTable.month = "Nov"
    elseif DateTable.month == 12 then
        DateTable.month = "Dec"
    end

    if CharSex == 1 then
        CharSex = "Neutral"
    elseif CharSex == 2 then
        CharSex = "Male"
    elseif CharSex == 3 then
        CharSex = "Female"
    end

    TimeDateString =
        (TimeString ..
        " - " .. DateTable.wday .. " " .. DateTable.day .. " " .. DateTable.month .. " " .. DateTable.year)
    NameClassRaceString = (name .. " - level " .. NewLevel .. " " .. race .. " " .. class)

    CharSex = sex
    CharName = name
    CharRace = race
    CharClass = class
end

local function testForMetaAchievement()
    local count = 0
    for k, v in pairs(IronManClassTable) do
        if v.male and v.female then
            count = count + 1
        end
    end
    if count == #IronManClassTable and not IronManGlobalVariables.metaClassesSexes then
        IronManGlobalVariables.metaClassesSexes = true
        TemporaryTable = {
            char = NameClassRaceString,
            timeDate = TimeDateString,
            title = StringList.ironManAllGenders,
            icon = ImageList.purple
        }
        table.insert(IronManCompleatedAchievements, 1, TemporaryTable)
        IronManMetaAchievementFrameTitleTextBoxText:SetText(StringList.ironManAllGenders)
        IronManMetaAchievementFrameleftTexture:SetTexture(ImageList.purple)
        IronManMetaAchievementFramerightTexture:SetTexture(ImageList.purple)
        IronManMetaAchievementFrameDateTextBoxText:SetText(TimeDateString)
        IronManMetaAchievementFrameNameTextBoxText:SetText(NameClassRaceString)
        IronManMetaAchievementFrame:Show()
        Screenshot()
    end
    count = 0
    TemporaryTable = {}

    for k, v in pairs(IronManClassTable) do
        if v.male or v.female then
            count = count + 1
        end
    end
    if count == #IronManClassTable and not IronManGlobalVariables.metaClasses then
        IronManGlobalVariables.metaClasses = true
        TemporaryTable = {
            char = NameClassRaceString,
            timeDate = TimeDateString,
            title = StringList.ironManAllClasses,
            icon = ImageList.blue
        }
        table.insert(IronManCompleatedAchievements, 1, TemporaryTable)
        IronManMetaAchievementFrameTitleTextBoxText:SetText(StringList.ironManAllClasses)
        IronManMetaAchievementFrameleftTexture:SetTexture(ImageList.blue)
        IronManMetaAchievementFramerightTexture:SetTexture(ImageList.blue)
        IronManMetaAchievementFrameDateTextBoxText:SetText(TimeDateString)
        IronManMetaAchievementFrameNameTextBoxText:SetText(NameClassRaceString)
        IronManMetaAchievementFrame:Show()
        Screenshot()
    end
    count = 0
    TemporaryTable = {}

    for k, v in pairs(IronManRaceTable) do
        if v.male or v.female then
            count = count + 1
        end
    end
    if count == #IronManRaceTable and not IronManGlobalVariables.metaRaces then
        IronManGlobalVariables.metaRaces = true
        TemporaryTable = {
            char = NameClassRaceString,
            timeDate = TimeDateString,
            title = StringList.ironManAllRaces,
            icon = ImageList.orange
        }
        table.insert(IronManCompleatedAchievements, 1, TemporaryTable)
        IronManMetaAchievementFrameTitleTextBoxText:SetText(StringList.ironManAllRaces)
        IronManMetaAchievementFrameleftTexture:SetTexture(ImageList.orange)
        IronManMetaAchievementFramerightTexture:SetTexture(ImageList.orange)
        IronManMetaAchievementFrameDateTextBoxText:SetText(TimeDateString)
        IronManMetaAchievementFrameNameTextBoxText:SetText(NameClassRaceString)
        IronManMetaAchievementFrame:Show()
        Screenshot()
    end
    count = 0
    TemporaryTable = {}

    if IronManGlobalVariables.metaJumps then -- jumps
        IronManGlobalVariables.metaJumps = false
        TemporaryTable = {
            char = NameClassRaceString,
            timeDate = TimeDateString,
            title = StringList.ironManJumps,
            icon = ImageList.green
        }
        table.insert(IronManCompleatedAchievements, 1, TemporaryTable)
        IronManMetaAchievementFrameTitleTextBoxText:SetText(StringList.ironManJumps)
        IronManMetaAchievementFrameleftTexture:SetTexture(ImageList.green)
        IronManMetaAchievementFramerightTexture:SetTexture(ImageList.green)
        IronManMetaAchievementFrameDateTextBoxText:SetText(TimeDateString)
        IronManMetaAchievementFrameNameTextBoxText:SetText(NameClassRaceString)
        IronManMetaAchievementFrame:Show()
        IronManGlobalVariables.metaJumps = false
        Screenshot()
    end
    count = 0
    TemporaryTable = {}

    if IronManGlobalVariables.totalNumberOfFails == NumberList.maxFails and not IronManGlobalVariables.metaFails then -- fails
        IronManGlobalVariables.metaFails = true
        TemporaryTable = {
            char = NameClassRaceString,
            timeDate = TimeDateString,
            title = StringList.ironManFails,
            icon = ImageList.grey
        }
        table.insert(IronManCompleatedAchievements, 1, TemporaryTable)
        IronManMetaAchievementFrameTitleTextBoxText:SetText(StringList.ironManFails)
        IronManMetaAchievementFrameleftTexture:SetTexture(ImageList.grey)
        IronManMetaAchievementFramerightTexture:SetTexture(ImageList.grey)
        IronManMetaAchievementFrameDateTextBoxText:SetText(TimeDateString)
        IronManMetaAchievementFrameNameTextBoxText:SetText(NameClassRaceString)
        IronManMetaAchievementFrame:Show()
        Screenshot()
    end
    count = 0
    TemporaryTable = {}

    if IronManGlobalVariables.totalNumberOfDeaths == NumberList.maxDeaths and not IronManGlobalVariables.metaDeaths then -- deaths
        IronManGlobalVariables.metaDeaths = true
        TemporaryTable = {
            char = NameClassRaceString,
            timeDate = TimeDateString,
            title = StringList.ironManDeaths,
            icon = ImageList.grey
        }
        table.insert(IronManCompleatedAchievements, 1, TemporaryTable)
        IronManMetaAchievementFrameTitleTextBoxText:SetText(StringList.ironManDeaths)
        IronManMetaAchievementFrameleftTexture:SetTexture(ImageList.grey)
        IronManMetaAchievementFramerightTexture:SetTexture(ImageList.grey)
        IronManMetaAchievementFrameDateTextBoxText:SetText(TimeDateString)
        IronManMetaAchievementFrameNameTextBoxText:SetText(NameClassRaceString)
        IronManMetaAchievementFrame:Show()
        Screenshot()
    end
    count = 0
    TemporaryTable = {}
    updateAchievementList()
end

local function maxLevelReached()
    TemporaryTable = {
        timeDate = TimeDateString,
        char = NameClassRaceString,
        title = StringList.ironManSuccess,
        icon = ImageList.goldRed
    }
    table.insert(IronManCompleatedAchievements, 1, TemporaryTable)
    TemporaryTable = {}
    IronManAchievementFrame:ClearAllPoints()
    IronManAchievementFrame:SetPoint("CENTER", 0, NumberList.achievementFramePositionY)
    IronManMetaAchievementFrame:ClearAllPoints()
    IronManMetaAchievementFrame:SetPoint("TOP", IronManAchievementFrame, "BOTTOM", 0, NumberList.metaAchievementTopGap)
    IronManAchievementFrameTitleTextBoxText:SetText(StringList.ironManSuccess)
    IronManAchievementFrameNameTextBoxText:SetText(NameClassRaceString)
    IronManAchievementFrameDateTextBoxText:SetText(TimeDateString)
    IronManMetaAchievementFrameNameTextBoxText:SetText(NameClassRaceString)
    IronManMetaAchievementFrameDateTextBoxText:SetText(TimeDateString)
    IronManAchievementFrame:Show()
    Screenshot()

    updateClassRaceTables()
    testForMetaAchievement()
    updateAchievementList()
    IronManCharacterVariables.isAttemptingIronMan = false
    IronManCharacterVariables.completedIronMan = true
    IronManCharacterVariables.failedIronMan = false
end

local function checkClassLevel() -- need to test class level
    local characterGUID = UnitGUID("player")
    local class, _, race = GetPlayerInfoByGUID(characterGUID)
    local level = UnitLevel("player")
    for k, v in pairs(IronManClassTable) do
        if v.class == class then
            if v.startingLevel == level then
                -- print("class pass", race, level) -- debug --
                return true
            else
                -- print("class fail", race, level) -- debug --
                return false
            end
        end
    end
end

local function checkRaceLevel() -- need to test race level
    local characterGUID = UnitGUID("player")
    local class, _, race = GetPlayerInfoByGUID(characterGUID)
    local level = UnitLevel("player")
    for k, v in pairs(IronManRaceTable) do
        if v.race == race then
            if v.startingLevel == level then
                -- print("race pass", race, level) -- debug --
                return true
            else
                -- print("race fail", race, level) -- debug --
                return false
            end
        end
    end
end

local function testPlayerStatDeath() -- check player death stat
    for _, catid in ipairs(GetStatisticsCategoryList()) do -- loop through all achievements
        for i = 1, GetCategoryNumAchievements(catid, true) do -- get the current categoryID
            local achid, achname = GetAchievementInfo(catid, i) -- get achievement name and ID
            if achid == 60 then -- acheivementID 60 is for the number of deaths
                if GetStatistic(achid) == 0 then -- if deaths equals zero
                    print(("(%d) %s: %d"):format(achid, achname, GetStatistic(achid))) -- debug --
                    return true
                else
                    return false
                end
            end
        end
    end
end

-- unequip all items excluding main and off hand items
-- code chunk kindly provided by SDPhantom https://www.wowinterface.com/forums/member.php?u=34145
local function UnequipAll()
    if #InventorySlots <= 0 then
        return
    end
    -- Sanity check
    local slotindex = 1

    ClearCursor() -- Make sure the cursor isn't holding anything, otherwise we might accidentally issue an item swap instead of an unequip
    for bag = NUM_BAG_SLOTS or NUM_TOTAL_EQUIPPED_BAG_SLOTS, 0, -1 do -- CE and Wrath use NUM_BAG_SLOTS, DF uses NUM_TOTAL_EQUIPPED_BAG_SLOTS
        local free, type = (C_Container or _G).GetContainerNumFreeSlots(bag) -- C_Container is used in Wrath and DF, CE still has this in _G
        free = (type == 0 and free or 0) -- Uses a quirk with this style of condition, only process bags with no item type restriction with fallback to zero if no bag is there (if free is nil, it'll fallback to zero even if the condition is true)

        for _ = 1, free do -- Variable is unused, we just need to loop for every free slot we see
            local invslot = InventorySlots[slotindex] -- Cache slot mapped to current index
            while not GetInventoryItemID("player", invslot) do -- Loop if no item in slot and until we find one
                if slotindex < #InventorySlots then
                    slotindex = slotindex + 1
                else
                    return
                end
                -- Increment to next index or stop when we have no more inventory slots to process
                invslot = InventorySlots[slotindex] -- Update to new slot
            end

            -- This pair is a complete operation, cursor is expected to be clear by the time both of these lines have run
            PickupInventoryItem(invslot); -- debug must use ; here as it precedes a new line that starts with a parentheses
            (bag == 0 and PutItemInBackpack or PutItemInBag)((C_Container or _G).ContainerIDToInventoryID(bag)) -- First set of parenthesis chooses function to run before calling it, PutItemInBackpack() doesn't accept any args, so ContainerIDToInventoryID() is safe to run regardless

            if slotindex < #InventorySlots then -- Increment to next index or stop when we have no more inventory slots to process
                slotindex = slotindex + 1
            else
                return
            end
        end
    end
end

-- ===================================================================================================
-- ===================================================================================================
-- ===========================         Hook Functions          ===========================
-- ===================================================================================================
-- ===================================================================================================

local function myHookFunctions()
    -- hook jump function
    hooksecurefunc(
        "JumpOrAscendStart",
        function()
            if IsFlying() or IsSwimming() or IsMounted() or IsFalling() then
                return
            else
                IronManGlobalVariables.totalNumberOfJumps = IronManGlobalVariables.totalNumberOfJumps + 1
                if IronManGlobalVariables.totalNumberOfJumps == NumberList.maxJumps then
                    IronManGlobalVariables.metaJumps = true
                    RequestTimePlayed()
                end
            end
        end
    )
    -- hook character equipment manager button
    hooksecurefunc(
        "EquipmentFlyoutPopoutButton_ShowAll",
        function()
            ToggleCharacter("PaperDollFrame")
        end
    )
    -- hook collections button
    hooksecurefunc(
        "CollectionsJournal_UpdateSelectedTab",
        function()
            print("collections")
            if CollectionsJournal:IsVisible() then
                CollectionsJournal:Hide()
            end
        end
    )
    -- not hooks but better grouped here
    -- disable lfg button
    local functionIWantToDisable1, isFunctionStillRelevant1 = PVEFrame_ToggleFrame, false
    function PVEFrame_ToggleFrame()
        if isFunctionStillRelevant1 then
            return functionIWantToDisable1()
        end
    end
    -- disable guild and communities button
    local functionIWantToDisable2, isFunctionStillRelevant2 = ToggleCommunitiesFrame, false
    function ToggleCommunitiesFrame()
        if isFunctionStillRelevant2 then
            return functionIWantToDisable2()
        end
    end
    -- disable the macro frame
    local functionIWantToDisable3, isFunctionStillRelevant3 = ShowMacroFrame, false
    function ShowMacroFrame()
        if isFunctionStillRelevant3 then
            return functionIWantToDisable3()
        end
    end
end

-- ===================================================================================================
-- ===================================================================================================
-- ===========================         Frames          ===========================
-- ===================================================================================================
-- ===================================================================================================

local IronManSelectionFrame = CreateFrame("Frame", "IronManSelectionFrame", UIParent, "BackdropTemplate")
IronManSelectionFrame:SetBackdrop(
    {
        bgFile = ImageList.IMBG,
        insets = nil,
        tileSize = nil,
        tile = false,
        edgeFile = nil,
        edgeSize = nil
    }
)
IronManSelectionFrame:SetSize(NumberList.selectionFrameWidth, NumberList.selectionFrameHeight)
IronManSelectionFrame:SetScale(NumberList.selectionFrameScale)
IronManSelectionFrame:ClearAllPoints()
IronManSelectionFrame:SetPoint("CENTER", 0, NumberList.selectionFramePositionY)
--[[
IronManSelectionFrame:EnableMouse(true) -- debug --
IronManSelectionFrame:SetMovable(true) -- debug --
IronManSelectionFrame:SetClampedToScreen(true) -- debug --
IronManSelectionFrame:RegisterForDrag("RightButton", "LeftButton") -- debug --
IronManSelectionFrame:SetScript("OnDragStart", IronManSelectionFrame.StartMoving) -- debug --
IronManSelectionFrame:SetScript("OnDragStop", IronManSelectionFrame.StopMovingOrSizing) -- debug --
--]]
IronManSelectionFrame:SetToplevel(true)
IronManSelectionFrame:SetScript(
    "OnEvent",
    function(self, event, ...)
        if event == "PLAYER_LOGIN" then
            print(StringList.ironManLoad)
            initialiseTables()
            if IronManCharacterVariables.isNew then -- brand new toon
                IronManCharacterVariables.logoutLevel = UnitLevel("player")
                RequestTimePlayed() -- need this to test if character is brand new
            elseif
                IronManCharacterVariables.isAttemptingIronMan and
                    IronManCharacterVariables.logoutLevel ~= UnitLevel("player")
             then -- if levels do not match
                failedIronManChallenge("Boosted")
            elseif IronManCharacterVariables.isAttemptingIronMan and IsCharacterNewlyBoosted() then -- if boosted
                failedIronManChallenge("Boosted")
            elseif IronManCharacterVariables.isAttemptingIronMan then
                IronManSelectionFrame:Show()
                C_EquipmentSet.UseEquipmentSet(0)
            else
                unregisterEventsIfNotRequired()
                IronManSelectionFrame:Hide()
            end
            if #IronManCompleatedAchievements ~= 0 then
                updateAchievementList()
            end
        elseif event == "PLAYER_LOGOUT" then
            IronManCharacterVariables.logoutLevel = UnitLevel("player")
        elseif event == "TIME_PLAYED_MSG" then
            TimeString = ...
            -- print("time played", TimeString) -- debug --
            if IronManCharacterVariables.isNew then
                if TimeString < 30 then
                    if IsCharacterNewlyBoosted() then
                        failedIronManChallenge("Boosted")
                    else
                        IronManSelectionFrameselectionButton:SetText(StringList.acceptChallenge)
                        IronManSelectionFramecloseButton:SetText(StringList.doNotAcceptChallenge)
                    end
                else
                    IronManSelectionFrame:Hide()
                end
                IronManCharacterVariables.isNew = false
            end
            if IronManCharacterVariables.completedIronMan then
                buildStrings()
                maxLevelReached()
            end
            if IronManGlobalVariables.metaJumps then
                NewLevel = UnitLevel("player")
                buildStrings()
                testForMetaAchievement()
            end
        elseif event == "PLAYER_DEAD" then
            failedIronManChallenge("Died")
            RequestTimePlayed()
            buildStrings()
            testForMetaAchievement()
        elseif event == "PLAYER_LEVEL_CHANGED" then
            local oldLevel, newLevel = ...
            local levelChangeIncrease = newLevel - oldLevel
            IronManCharacterVariables.logoutLevel = newLevel
            if levelChangeIncrease > 1 then
                failedIronManChallenge("Boosted")
            elseif IsCharacterNewlyBoosted() then
                failedIronManChallenge("Boosted")
            elseif IronManSelectionFrame:IsVisible() then
                failedIronManChallenge("Boosted")
--            elseif not testPlayerStatDeath() then -- just to ensure no deaths
--                failedIronManChallenge("Died")
            elseif newLevel == NumberList.maxLevel then
                -- print("new level", newLevel, "  max level", NumberList.maxLevel) -- debug --
                IronManCharacterVariables.completedIronMan = true
                NewLevel = newLevel
                RequestTimePlayed()
                buildStrings()
                maxLevelReached()
            end
        elseif event == "PLAYER_EQUIPMENT_CHANGED" then
            --[[ -- for equipping grey items only -- debug -- remove
            if itemQuality ~= 0 then -- if item is not grey
                C_EquipmentSet.UseEquipmentSet(0)
            else
                for k, v in pairs(C_EquipmentSet.GetEquipmentSetIDs()) do
                    C_EquipmentSet.DeleteEquipmentSet(v) -- delete all equipment sets
                end
                C_EquipmentSet.CreateEquipmentSet("IronMan", 1035053) -- create new equipment set
            end

            local itemQuality = GetInventoryItemQuality("player", equipmentSlot)
            --if not IronManCharacterVariables.isAttemptingIronMan then
            C_EquipmentSet.UseEquipmentSet(0)
--]]
            local equipmentSlot = ...
            local charGUID = UnitGUID("player")
            local class = GetPlayerInfoByGUID(charGUID)

            if
                equipmentSlot == 16 and (class == "Evoker" or class == "Death Knight") and
                    not IronManCharacterVariables.evokerDeathknightWeaponEquipped
             then
                IronManCharacterVariables.evokerDeathknightWeaponEquipped = true
            else
                C_EquipmentSet.UseEquipmentSet(0)
            end
        elseif event == "PLAYER_ENTERING_WORLD" then -- when entering an instance
            C_Timer.After(
                0.5,
                function()
                    if IsInInstance() then -- show fail warning
                        IronManWarningFrameDetailsTextBoxText:SetText(StringList.ironManWarningInstance)
                        IronManWarningFrame:Show()
                        C_Timer.After(
                            7, -- debug -- may extend
                            function()
                                if IsInInstance() then -- if still in instance
                                    IronManWarningFrame:Hide()
                                    failedIronManChallenge("EnteredInstance")
                                else
                                    IronManWarningFrame:Hide()
                                end
                            end
                        )
                    end
                end
            )
        elseif event == "UPDATE_SHAPESHIFT_FORM" then
            shapeForm = GetShapeshiftFormID()
            if shapeForm == 3 or shapeForm == 4 or shapeForm == 16 or shapeForm == 27 or shapeForm == 29 then
                IronManWarningFrameDetailsTextBoxText:SetText(StringList.ironManShape)
                IronManWarningFrame:Show() -- show fail warning
                C_Timer.After(
                    3,
                    function()
                        shapeForm = GetShapeshiftFormID() -- check if current form is prohibited
                        if shapeForm == 3 or shapeForm == 4 or shapeForm == 16 or shapeForm == 27 or shapeForm == 29 then
                            IronManWarningFrame:Hide()
                            failedIronManChallenge("ShapeShifted")
                        else
                            IronManWarningFrame:Hide()
                        end
                    end
                )
            end
        elseif event == "PET_BATTLE_OPENING_DONE" then
            IronManWarningFrameDetailsTextBoxText:SetText(StringList.ironManWarningPetBattle)
            IronManWarningFrame:Show() -- show fail warning
            C_Timer.After(
                10,
                function()
                    if C_PetBattles.IsInBattle() then -- if still in pet battle
                        IronManWarningFrame:Hide()
                        failedIronManChallenge("PetBattle")
                    else
                        IronManWarningFrame:Hide()
                    end
                end
            )
        elseif
            event == "GARRISON_SHIPYARD_NPC_OPENED" or event == "GARRISON_ARCHITECT_OPENED" or
                event == "ADVENTURE_MAP_OPEN" or
                event == "GARRISON_MISSION_NPC_OPENED" or
                event == "PLAYER_INTERACTION_MANAGER_FRAME_SHOW"
         then -- hide any of the open table frames
            GarrisonMissionFrame:Hide()
            GarrisonShipyardFrame:Hide()
            GarrisonBuildingFrame:Hide()
            OrderHallMissionFrame:Hide()
            BFAMissionFrame:Hide()
            CovenantMissionFrame:Hide()
        elseif event == "UNIT_SPELLCAST_SUCCEEDED" then
            _, _, spellID = ...
            mountID = C_MountJournal.GetMountFromSpell(spellID) -- each mount has a summon spell
            for k, v in pairs(C_MountJournal.GetMountIDs()) do -- scan the the mount list
                if v == mountID then -- when the mountID matches
                    Dismount()
                    break
                end
            end
            if spellID == TravelSpellID then -- if a travel spell is successfully cast
                failedIronManChallenge("Travel")
            end
        elseif event == "UNIT_SPELLCAST_SENT" then -- test if casting hearthstone, portal or teleport spell
            unit, _, _, spellID = ...
            if unit == "player" then
                for k, v in pairs(travelList) do
                    if spellID == v then
                        TravelSpellID = spellID -- casting a travel spell
                        IronManWarningFrameDetailsTextBoxText:SetText(StringList.ironManWarningTravel)
                        IronManWarningFrame:Show() -- show warning frame
                        C_Timer.After( -- if has not stopped cast
                            6,
                            function()
                                IronManWarningFrame:Hide()
                                failedIronManChallenge("Travel")
                            end
                        )
                    end
                end
            end
        elseif event == "TRADE_SHOW" then -- block all trades
            IronManWarningFrame:Show()
            C_Timer.After(
                0.5,
                function()
                    CloseTrade()
                end
            )
            CloseTrade()
        elseif event == "TAXIMAP_OPENED" then
            FlightMapFrame:Hide()
        elseif event == "AUCTION_HOUSE_SHOW" then
            C_AuctionHouse.CloseAuctionHouse()
        elseif event == "TRAINER_SHOW" then
            CloseTrainer()
        elseif event == "GUILD_INVITE_REQUEST" then
            DeclineGuild()
        elseif event == "PARTY_INVITE_REQUEST" then
            DeclineGroup()
        elseif event == "MERCHANT_SHOW" then
            closeVendorIfIsAGuildMaster()
        elseif event == "GOSSIP_SHOW" then
            closeVendorIfIsAGuildMaster()
        elseif event == "PETITION_SHOW" then
            ClosePetition()
        elseif event == "GROUP_FORMED" then
            failedIronManChallenge("JoinedGroup")
        elseif event == "EQUIP_BIND_CONFIRM" then
            ClearCursor()
        elseif event == "MAIL_SHOW" then
            CloseMail()
        end
    end
)
IronManSelectionFrame:RegisterEvent("PLAYER_LOGIN") -- fires when a player logs in or reloads
IronManSelectionFrame:RegisterEvent("ADVENTURE_MAP_OPEN") -- fires when Legion BFA SLands table opens
IronManSelectionFrame:RegisterEvent("AUCTION_HOUSE_SHOW") -- fires when the auction house opens
IronManSelectionFrame:RegisterEvent("EQUIP_BIND_CONFIRM") -- fires when an equip confirm box is shown
IronManSelectionFrame:RegisterEvent("GARRISON_ARCHITECT_OPENED") -- fires when garrison architect table opens
IronManSelectionFrame:RegisterEvent("GARRISON_MISSION_NPC_OPENED") -- fires when garrison table opens
IronManSelectionFrame:RegisterEvent("GARRISON_SHIPYARD_NPC_OPENED") -- fires when garrison shipyard table opens
IronManSelectionFrame:RegisterEvent("GOSSIP_SHOW") -- fires when you see a gossip window
IronManSelectionFrame:RegisterEvent("GROUP_FORMED") -- fires when joined group
IronManSelectionFrame:RegisterEvent("GUILD_INVITE_REQUEST") -- fires when a charater receives a guild invite
IronManSelectionFrame:RegisterEvent("MAIL_SHOW") -- fires when the mail box is opened
IronManSelectionFrame:RegisterEvent("MERCHANT_SHOW") -- fires you open a vendor
IronManSelectionFrame:RegisterEvent("PARTY_INVITE_REQUEST") -- fires when a character receives a group invite
IronManSelectionFrame:RegisterEvent("PET_BATTLE_OPENING_DONE") -- fires when the pet battle setup ends
IronManSelectionFrame:RegisterEvent("PETITION_SHOW") -- fires when guild petition is shown
IronManSelectionFrame:RegisterEvent("PLAYER_DEAD") -- fires when a character dies
IronManSelectionFrame:RegisterEvent("PLAYER_ENTERING_WORLD") -- fires when the player sees a loading screen
IronManSelectionFrame:RegisterEvent("PLAYER_EQUIPMENT_CHANGED") -- fires when a character changes equipment, returns equipmentSlot
IronManSelectionFrame:RegisterEvent("PLAYER_INTERACTION_MANAGER_FRAME_SHOW") -- fires when the garrison architect table opens
IronManSelectionFrame:RegisterEvent("PLAYER_LEVEL_CHANGED") -- fires when a character levels up, returns oldLevel, newLevel
IronManSelectionFrame:RegisterEvent("PLAYER_LOGOUT") -- fires when a character logs out
IronManSelectionFrame:RegisterEvent("TAXIMAP_OPENED") -- fires when flight master map opens
IronManSelectionFrame:RegisterEvent("TIME_PLAYED_MSG") -- fires when a timeplayed is requested, returns totalTimePlayed in seconds
IronManSelectionFrame:RegisterEvent("TRADE_SHOW") -- fires when a trade window is opened
IronManSelectionFrame:RegisterEvent("TRAINER_SHOW") -- fires when a trainer window is opened
IronManSelectionFrame:RegisterEvent("UNIT_SPELLCAST_SENT") -- fires when a spell is sent
IronManSelectionFrame:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED") -- fires when a spell has been cast
IronManSelectionFrame:RegisterEvent("UPDATE_SHAPESHIFT_FORM") -- fires when a druid or shaman shapeshifts

IronManSelectionFrame.iconTexture = IronManSelectionFrame:CreateTexture("IronManSelectionFrameiconTexture")
IronManSelectionFrameiconTexture:SetScale(NumberList.iconTextureScale)
IronManSelectionFrameiconTexture:SetTexture(ImageList.goldRed)
IronManSelectionFrameiconTexture:SetPoint("LEFT", IronManSelectionFrame)

IronManSelectionFrame.selectionButton =
    CreateFrame(
    "Button",
    "IronManSelectionFrameselectionButton",
    IronManSelectionFrame,
    "SecureActionButtonTemplate, SharedButtonTemplate"
)
IronManSelectionFrameselectionButton:SetSize(NumberList.selectionButtonWidth, NumberList.selectionButtonHeight)
IronManSelectionFrameselectionButton:SetPoint(
    "CENTER",
    IronManSelectionFrame,
    "LEFT",
    NumberList.buttonSpacerLeft,
    NumberList.buttonSpacerTop
)
IronManSelectionFrameselectionButton:SetText(StringList.continueChallenge)
IronManSelectionFrameselectionButton:SetScript(
    "OnClick",
    function()
        if not IronManCharacterVariables.isAttemptingIronMan then
            UnequipAll()
            C_Timer.After(
                0.6,
                function()
                    C_EquipmentSet.CreateEquipmentSet("IronMan", 1035053) -- create new equipment set
                end
            )
            IronManCharacterVariables.isAttemptingIronMan = true
        end
        IronManSelectionFrame:Hide()
        myHookFunctions()
        C_EquipmentSet.UseEquipmentSet(0)
        -- PlaySoundFile(SoundList.suitUp, "Master") -- debug --
    end
)

IronManSelectionFrame.closeButton =
    CreateFrame("Button", "IronManSelectionFramecloseButton", IronManSelectionFrame, "SharedButtonTemplate")
IronManSelectionFramecloseButton:SetSize(NumberList.selectionButtonWidth, NumberList.selectionButtonHeight)
IronManSelectionFramecloseButton:SetPoint(
    "TOP",
    IronManSelectionFrameselectionButton,
    "BOTTOM",
    0,
    NumberList.buttonSpacerBottom
)
IronManSelectionFramecloseButton:SetText(StringList.doNotContinueChallenge)
IronManSelectionFramecloseButton:SetScript(
    "OnClick",
    function()
        IronManSelectionFrame:Hide()
        IronManCharacterVariables.isAttemptingIronMan = false
        IronManCharacterVariables.completedIronMan = false
        unregisterEventsIfNotRequired()
        -- PlaySoundFile(SoundList.nextTime, "Master") -- debug --
    end
)

IronManSelectionFrame.TextBox = CreateFrame("Frame", "IronManSelectionFrameTextBox", IronManSelectionFrame)
IronManSelectionFrameTextBox:SetSize(150, NumberList.textBoxHeight)
IronManSelectionFrameTextBox:SetPoint("LEFT", IronManSelectionFrameselectionButton, "RIGHT", 15, 0)

IronManSelectionFrameTextBox.Text = IronManSelectionFrameTextBox:CreateFontString("IronManSelectionFrameTextBoxText")
IronManSelectionFrameTextBoxText:SetAllPoints(IronManSelectionFrameTextBox)
IronManSelectionFrameTextBoxText:SetFontObject(FontList.rulesFont)
IronManSelectionFrameTextBoxText:SetText(StringList.ironManRulesShort)

IronManSelectionFrame.TextBoxFail = CreateFrame("Frame", "IronManSelectionFrameTextBoxFail", IronManSelectionFrame)
IronManSelectionFrameTextBoxFail:SetSize(NumberList.failTextBoxWidth, NumberList.failTextBoxHeight)
IronManSelectionFrameTextBoxFail:SetPoint(
    "TOPRIGHT",
    IronManSelectionFrame,
    "TOPRIGHT",
    NumberList.failTextBoxSpacerLeft,
    NumberList.failTextBoxSpacerTop
)
IronManSelectionFrameTextBoxFail:Hide()

IronManSelectionFrameTextBoxFail.Text =
    IronManSelectionFrameTextBoxFail:CreateFontString("IronManSelectionFrameTextBoxFailText")
IronManSelectionFrameTextBoxFailText:SetAllPoints(IronManSelectionFrameTextBoxFail)
IronManSelectionFrameTextBoxFailText:SetFontObject(FontList.failFont)

IronManSelectionFrame.rulesButton =
    CreateFrame("Button", "IronManSelectionFramerulesButton", IronManSelectionFrame, "SharedButtonTemplate")
IronManSelectionFramerulesButton:SetSize(NumberList.selectionButtonWidth, NumberList.selectionButtonHeight)
IronManSelectionFramerulesButton:SetPoint("LEFT", IronManSelectionFramecloseButton, "RIGHT", 0, 0)
IronManSelectionFramerulesButton:SetText(StringList.rulesButton)
IronManSelectionFramerulesButton:SetScript(
    "OnClick",
    function()
        InterfaceOptionsFrame_OpenToCategory(IronManInterfaceFrame)
    end
)

-- IronManSelectionFrame:Show() -- debug --

-- ===================================================================================================
-- ===================================================================================================
-- ===========================         max level achievement frame          ===========================
-- ===================================================================================================
-- ===================================================================================================

local IronManAchievementFrame = CreateFrame("Frame", "IronManAchievementFrame", UIParent, "BackdropTemplate")
IronManAchievementFrame:SetBackdrop(
    {
        bgFile = ImageList.IMBG,
        insets = nil,
        tileSize = nil,
        tile = false,
        edgeFile = nil,
        edgeSize = nil
    }
)
IronManAchievementFrame:SetSize(NumberList.achievementFrameWidth, NumberList.achievementFrameHeight)
IronManAchievementFrame:ClearAllPoints()
IronManAchievementFrame:SetPoint("CENTER", 0, NumberList.achievementFramePositionY)
--[[
IronManAchievementFrame:EnableMouse(true) -- debug --
IronManAchievementFrame:SetMovable(true) -- debug --
IronManAchievementFrame:SetClampedToScreen(true) -- debug --
IronManAchievementFrame:RegisterForDrag("RightButton", "LeftButton") -- debug --
IronManAchievementFrame:SetScript("OnDragStart", IronManAchievementFrame.StartMoving) -- debug --
IronManAchievementFrame:SetScript("OnDragStop", IronManAchievementFrame.StopMovingOrSizing) -- debug --
--]]
IronManAchievementFrame:SetToplevel(true)
IronManAchievementFrame:Hide()

IronManAchievementFrame.CloseButton =
    CreateFrame(
    "Button",
    "IronManAchievementFrameCloseButton",
    IronManAchievementFrame,
    "SocialToastCloseButtonTemplate"
)
IronManAchievementFrameCloseButton:SetPoint(
    "TOPRIGHT",
    IronManAchievementFrame,
    "TOPRIGHT",
    NumberList.achievementCloseButtonX,
    NumberList.achievementCloseButtonY
)
IronManAchievementFrameCloseButton:SetScript(
    "OnEnter",
    function()
    end
)
IronManAchievementFrameCloseButton:SetScript(
    "OnLeave",
    function()
    end
)

IronManAchievementFrame.TitleTextBox =
    CreateFrame("Frame", "IronManAchievementFrameTitleTextBox", IronManAchievementFrame)
IronManAchievementFrameTitleTextBox:SetSize(NumberList.titleTextBoxWidth, NumberList.titleTextBoxHeight)
IronManAchievementFrameTitleTextBox:SetPoint("TOP", IronManAchievementFrame, "TOP", 0, NumberList.titleTextBoxTopGap)

IronManAchievementFrameTitleTextBox.Text =
    IronManAchievementFrameTitleTextBox:CreateFontString("IronManAchievementFrameTitleTextBoxText")
IronManAchievementFrameTitleTextBoxText:SetAllPoints(IronManAchievementFrameTitleTextBox)
IronManAchievementFrameTitleTextBoxText:SetFontObject(FontList.titleFont)
IronManAchievementFrameTitleTextBoxText:SetText(StringList.ironManSuccess)

IronManAchievementFrame.leftTexture = IronManAchievementFrame:CreateTexture("IronManAchievementFrameleftTexture")
IronManAchievementFrameleftTexture:SetScale(NumberList.iconAchievementTextureScale)
IronManAchievementFrameleftTexture:SetTexture(ImageList.goldRed)
IronManAchievementFrameleftTexture:SetPoint(
    "CENTER",
    IronManAchievementFrameTitleTextBox,
    "LEFT",
    NumberList.bottomLeftTextureX,
    0
)

IronManAchievementFrame.rightTexture = IronManAchievementFrame:CreateTexture("IronManAchievementFramerightTexture")
IronManAchievementFramerightTexture:SetScale(NumberList.iconAchievementTextureScale)
IronManAchievementFramerightTexture:SetTexture(ImageList.goldRed)
IronManAchievementFramerightTexture:SetPoint(
    "CENTER",
    IronManAchievementFrameTitleTextBox,
    "RIGHT",
    NumberList.bottomRightTextureX,
    0
)

IronManAchievementFrame.DateTextBox =
    CreateFrame("Frame", "IronManAchievementFrameDateTextBox", IronManAchievementFrame)
IronManAchievementFrameDateTextBox:SetSize(NumberList.achievementFrameWidth, NumberList.dateNameTextBoxHeight)
IronManAchievementFrameDateTextBox:SetPoint(
    "BOTTOM",
    IronManAchievementFrame,
    "BOTTOM",
    0,
    NumberList.dateTextBoxBottomGap
)

IronManAchievementFrameDateTextBox.Text =
    IronManAchievementFrameDateTextBox:CreateFontString("IronManAchievementFrameDateTextBoxText")
IronManAchievementFrameDateTextBoxText:SetAllPoints(IronManAchievementFrameDateTextBox)
IronManAchievementFrameDateTextBoxText:SetFontObject(FontList.nameDateFont)

IronManAchievementFrame.NameTextBox =
    CreateFrame("Frame", "IronManAchievementFrameNameTextBox", IronManAchievementFrame)
IronManAchievementFrameNameTextBox:SetSize(NumberList.achievementFrameWidth, NumberList.dateNameTextBoxHeight)
IronManAchievementFrameNameTextBox:SetPoint(
    "BOTTOM",
    IronManAchievementFrameDateTextBox,
    "TOP",
    0,
    NumberList.nameTextBoxBottomGap
)

IronManAchievementFrameNameTextBox.Text =
    IronManAchievementFrameNameTextBox:CreateFontString("IronManAchievementFrameNameTextBoxText")
IronManAchievementFrameNameTextBoxText:SetAllPoints(IronManAchievementFrameNameTextBox)
IronManAchievementFrameNameTextBoxText:SetFontObject(FontList.nameDateFont)

-- ===================================================================================================
-- ===================================================================================================
-- ===========================         meta achievement frame          ===========================
-- ===================================================================================================
-- ===================================================================================================

local IronManMetaAchievementFrame = CreateFrame("Frame", "IronManMetaAchievementFrame", UIParent, "BackdropTemplate")
IronManMetaAchievementFrame:SetBackdrop(
    {
        bgFile = ImageList.IMBG,
        insets = nil,
        tileSize = nil,
        tile = false,
        edgeFile = nil,
        edgeSize = nil
    }
)
IronManMetaAchievementFrame:SetSize(NumberList.achievementFrameWidth, NumberList.achievementFrameHeight)
IronManMetaAchievementFrame:ClearAllPoints()
IronManMetaAchievementFrame:SetPoint("CENTER", 0, NumberList.achievementFramePositionY)
IronManMetaAchievementFrame:EnableMouse(true) -- debug --
IronManMetaAchievementFrame:SetMovable(true) -- debug --
IronManMetaAchievementFrame:SetClampedToScreen(true) -- debug --
IronManMetaAchievementFrame:RegisterForDrag("RightButton", "LeftButton") -- debug --
IronManMetaAchievementFrame:SetScript("OnDragStart", IronManMetaAchievementFrame.StartMoving) -- debug --
IronManMetaAchievementFrame:SetScript("OnDragStop", IronManMetaAchievementFrame.StopMovingOrSizing) -- debug --
IronManMetaAchievementFrame:SetToplevel(true)
IronManMetaAchievementFrame:Hide()

IronManMetaAchievementFrame.CloseButton =
    CreateFrame(
    "Button",
    "IronManMetaAchievementFrameCloseButton",
    IronManMetaAchievementFrame,
    "SocialToastCloseButtonTemplate"
)
IronManMetaAchievementFrameCloseButton:SetPoint(
    "TOPRIGHT",
    IronManMetaAchievementFrame,
    "TOPRIGHT",
    NumberList.achievementCloseButtonX,
    NumberList.achievementCloseButtonY
)
IronManMetaAchievementFrameCloseButton:SetScript(
    "OnEnter",
    function()
    end
)
IronManMetaAchievementFrameCloseButton:SetScript(
    "OnLeave",
    function()
    end
)

IronManMetaAchievementFrame.TitleTextBox =
    CreateFrame("Frame", "IronManMetaAchievementFrameTitleTextBox", IronManMetaAchievementFrame)
IronManMetaAchievementFrameTitleTextBox:SetSize(NumberList.titleTextBoxWidth, NumberList.titleTextBoxHeight)
IronManMetaAchievementFrameTitleTextBox:SetPoint(
    "TOP",
    IronManMetaAchievementFrame,
    "TOP",
    0,
    NumberList.titleTextBoxTopGap
)

IronManMetaAchievementFrameTitleTextBox.Text =
    IronManMetaAchievementFrameTitleTextBox:CreateFontString("IronManMetaAchievementFrameTitleTextBoxText")
IronManMetaAchievementFrameTitleTextBoxText:SetAllPoints(IronManMetaAchievementFrameTitleTextBox)
IronManMetaAchievementFrameTitleTextBoxText:SetFontObject(FontList.titleFont)
IronManMetaAchievementFrameTitleTextBoxText:SetText(StringList.ironManSuccess)

IronManMetaAchievementFrame.leftTexture =
    IronManMetaAchievementFrame:CreateTexture("IronManMetaAchievementFrameleftTexture")
IronManMetaAchievementFrameleftTexture:SetScale(NumberList.iconAchievementTextureScale)
IronManMetaAchievementFrameleftTexture:SetTexture(ImageList.goldRed)
IronManMetaAchievementFrameleftTexture:SetPoint(
    "CENTER",
    IronManMetaAchievementFrameTitleTextBox,
    "LEFT",
    NumberList.bottomLeftTextureX,
    0
)

IronManMetaAchievementFrame.rightTexture =
    IronManMetaAchievementFrame:CreateTexture("IronManMetaAchievementFramerightTexture")
IronManMetaAchievementFramerightTexture:SetScale(NumberList.iconAchievementTextureScale)
IronManMetaAchievementFramerightTexture:SetTexture(ImageList.goldRed)
IronManMetaAchievementFramerightTexture:SetPoint(
    "CENTER",
    IronManMetaAchievementFrameTitleTextBox,
    "RIGHT",
    NumberList.bottomRightTextureX,
    0
)

IronManMetaAchievementFrame.DateTextBox =
    CreateFrame("Frame", "IronManMetaAchievementFrameDateTextBox", IronManMetaAchievementFrame)
IronManMetaAchievementFrameDateTextBox:SetSize(NumberList.achievementFrameWidth, NumberList.dateNameTextBoxHeight)
IronManMetaAchievementFrameDateTextBox:SetPoint(
    "BOTTOM",
    IronManMetaAchievementFrame,
    "BOTTOM",
    0,
    NumberList.dateTextBoxBottomGap
)

IronManMetaAchievementFrameDateTextBox.Text =
    IronManMetaAchievementFrameDateTextBox:CreateFontString("IronManMetaAchievementFrameDateTextBoxText")
IronManMetaAchievementFrameDateTextBoxText:SetAllPoints(IronManMetaAchievementFrameDateTextBox)
IronManMetaAchievementFrameDateTextBoxText:SetFontObject(FontList.nameDateFont)

IronManMetaAchievementFrame.NameTextBox =
    CreateFrame("Frame", "IronManMetaAchievementFrameNameTextBox", IronManMetaAchievementFrame)
IronManMetaAchievementFrameNameTextBox:SetSize(NumberList.achievementFrameWidth, NumberList.dateNameTextBoxHeight)
IronManMetaAchievementFrameNameTextBox:SetPoint(
    "BOTTOM",
    IronManMetaAchievementFrameDateTextBox,
    "TOP",
    0,
    NumberList.nameTextBoxBottomGap
)

IronManMetaAchievementFrameNameTextBox.Text =
    IronManMetaAchievementFrameNameTextBox:CreateFontString("IronManMetaAchievementFrameNameTextBoxText")
IronManMetaAchievementFrameNameTextBoxText:SetAllPoints(IronManMetaAchievementFrameNameTextBox)
IronManMetaAchievementFrameNameTextBoxText:SetFontObject(FontList.nameDateFont)

-- ===================================================================================================
-- ===================================================================================================
-- ===========================         warning frame          ===========================
-- ===================================================================================================
-- ===================================================================================================

local IronManWarningFrame = CreateFrame("Frame", "IronManWarningFrame", UIParent, "BackdropTemplate")
IronManWarningFrame:SetBackdrop(
    {
        bgFile = ImageList.IMBG,
        insets = nil,
        tileSize = nil,
        tile = false,
        edgeFile = nil,
        edgeSize = nil
    }
)
IronManWarningFrame:SetSize(NumberList.achievementFrameWidth, NumberList.achievementFrameHeight)
IronManWarningFrame:ClearAllPoints()
IronManWarningFrame:SetPoint("CENTER", 0, NumberList.achievementFramePositionY)

IronManWarningFrame.CloseButton =
    CreateFrame("Button", "IronManWarningFrameCloseButton", IronManWarningFrame, "SocialToastCloseButtonTemplate")
IronManWarningFrameCloseButton:SetPoint(
    "TOPRIGHT",
    IronManWarningFrame,
    "TOPRIGHT",
    NumberList.achievementCloseButtonX,
    NumberList.achievementCloseButtonY
)
IronManWarningFrameCloseButton:SetScript(
    "OnEnter",
    function()
    end
)
IronManWarningFrameCloseButton:SetScript(
    "OnLeave",
    function()
    end
)
IronManWarningFrame.TitleTextBox = CreateFrame("Frame", "IronManWarningFrameTitleTextBox", IronManWarningFrame)
IronManWarningFrameTitleTextBox:SetSize(NumberList.titleTextBoxWidth, NumberList.titleTextBoxHeight)
IronManWarningFrameTitleTextBox:SetPoint("TOP", IronManWarningFrame, "TOP", 0, NumberList.titleTextBoxTopGap)

IronManWarningFrameTitleTextBox.Text =
    IronManWarningFrameTitleTextBox:CreateFontString("IronManWarningFrameTitleTextBoxText")
IronManWarningFrameTitleTextBoxText:SetAllPoints(IronManWarningFrameTitleTextBox)
IronManWarningFrameTitleTextBoxText:SetFontObject(FontList.titleFont)
IronManWarningFrameTitleTextBoxText:SetText(StringList.ironManWarning)

IronManWarningFrame.leftTexture = IronManWarningFrame:CreateTexture("IronManWarningFrameleftTexture")
IronManWarningFrameleftTexture:SetScale(NumberList.iconAchievementTextureScale)
IronManWarningFrameleftTexture:SetTexture(ImageList.goldRed)
IronManWarningFrameleftTexture:SetPoint(
    "CENTER",
    IronManWarningFrameTitleTextBox,
    "LEFT",
    NumberList.bottomLeftTextureX,
    0
)

IronManWarningFrame.rightTexture = IronManWarningFrame:CreateTexture("IronManWarningFramerightTexture")
IronManWarningFramerightTexture:SetScale(NumberList.iconAchievementTextureScale)
IronManWarningFramerightTexture:SetTexture(ImageList.goldRed)
IronManWarningFramerightTexture:SetPoint(
    "CENTER",
    IronManWarningFrameTitleTextBox,
    "RIGHT",
    NumberList.bottomRightTextureX,
    0
)

IronManWarningFrame.DetailsTextBox = CreateFrame("Frame", "IronManWarningFrameDetailsTextBox", IronManWarningFrame)
IronManWarningFrameDetailsTextBox:SetSize(NumberList.warningDetailsTextBoxWidth, NumberList.warningDetailsTextBoxHeight)
IronManWarningFrameDetailsTextBox:SetPoint("BOTTOM", IronManWarningFrame, "BOTTOM", 0, NumberList.dateTextBoxBottomGap)

IronManWarningFrameDetailsTextBox.Text =
    IronManWarningFrameDetailsTextBox:CreateFontString("IronManWarningFrameDetailsTextBoxText")
IronManWarningFrameDetailsTextBoxText:SetWordWrap(true)
IronManWarningFrameDetailsTextBoxText:SetAllPoints(IronManWarningFrameDetailsTextBox)
IronManWarningFrameDetailsTextBoxText:SetFontObject(FontList.nameDateFont)
IronManWarningFrameDetailsTextBoxText:SetText()
IronManWarningFrame:Hide()

-- ===================================================================================================
-- ===================================================================================================
-- ===========================         info panel          ===========================
-- ===================================================================================================
-- ===================================================================================================

local IronManInterfaceFrame = CreateFrame("Frame", "IronManInterfaceFrame", nil, "BackdropTemplate")
IronManInterfaceFrame:SetBackdrop(
    {
        bgFile = ImageList.IMBG,
        insets = nil,
        tileSize = nil,
        tile = false,
        edgeFile = nil,
        edgeSize = nil
    }
)
IronManInterfaceFrame.name = "Iron Man"
InterfaceOptions_AddCategory(IronManInterfaceFrame)

local IronManInterfaceTitleTexture = CreateFrame("Frame", "IronManInterfaceTitleTexture", IronManInterfaceFrame)
IronManInterfaceTitleTexture:SetSize(NumberList.interfaceTextureBoxSize, NumberList.interfaceTextureBoxSize)
IronManInterfaceTitleTexture:SetPoint("TOP", 0, NumberList.interfaceTextureGap)
IronManInterfaceTitleTexture.Texture = IronManInterfaceTitleTexture:CreateTexture(nil, "BACKGROUND")
IronManInterfaceTitleTexture.Texture:SetPoint("CENTER")
IronManInterfaceTitleTexture.Texture:SetScale(NumberList.interfaceTextureScale)
IronManInterfaceTitleTexture.Texture:SetTexture(ImageList.IMBanner)

IronManInterfaceFrame.leftTextBox = CreateFrame("Frame", "IronManInterfaceFrameleftTextBox", IronManInterfaceFrame)
IronManInterfaceFrameleftTextBox:SetSize(NumberList.rulessTextBoxLeftWidth, NumberList.rulesTextBoxLeftHeight)
IronManInterfaceFrameleftTextBox:SetPoint(
    "TOPLEFT",
    IronManInterfaceFrame,
    "TOPLEFT",
    NumberList.rulesTextBoxLeftTopGap,
    NumberList.rulesTextBoxLeftSideGap
)

IronManInterfaceFrameleftTextBox.Text =
    IronManInterfaceFrameleftTextBox:CreateFontString("IronManInterfaceFrameleftTextBoxText")
IronManInterfaceFrameleftTextBoxText:SetAllPoints(IronManInterfaceFrameleftTextBox)
IronManInterfaceFrameleftTextBoxText:SetFontObject(FontList.rulesFont)
IronManInterfaceFrameleftTextBoxText:SetText(StringList.ironManRules)

IronManInterfaceFrame.rightTextBox = CreateFrame("Frame", "IronManInterfaceFramerightTextBox", IronManInterfaceFrame)
IronManInterfaceFramerightTextBox:SetSize(NumberList.rulessTextBoxRightWidth, NumberList.rulesTextBoxRightHeight)
IronManInterfaceFramerightTextBox:SetPoint("TOPLEFT", IronManInterfaceFrameleftTextBox, "TOPRIGHT", 0, 0)

IronManInterfaceFramerightTextBox.Text =
    IronManInterfaceFramerightTextBox:CreateFontString("IronManInterfaceFramerighteftTextBoxText")
IronManInterfaceFramerighteftTextBoxText:SetWordWrap(true)
IronManInterfaceFramerighteftTextBoxText:SetAllPoints(IronManInterfaceFramerightTextBox)
IronManInterfaceFramerighteftTextBoxText:SetFontObject(FontList.rulesFont)
IronManInterfaceFramerighteftTextBoxText:SetText(StringList.ironManRulesDetails)

-- ===================================================================================================
-- ===================================================================================================
-- ===========================         achievement panel          ===========================
-- ===================================================================================================
-- ===================================================================================================

local IronManInterfaceFrameCompleted = CreateFrame("Frame", "IronManInterfaceFrameCompleted", nil, "BackdropTemplate")
IronManInterfaceFrameCompleted:SetBackdrop(
    {
        bgFile = ImageList.IMBG,
        insets = nil,
        tileSize = nil,
        tile = false,
        edgeFile = nil,
        edgeSize = nil
    }
)
IronManInterfaceFrameCompleted.name = "Completed"
IronManInterfaceFrameCompleted.parent = "Iron Man"
InterfaceOptions_AddCategory(IronManInterfaceFrameCompleted, IronManInterfaceFrame)

local IronManInterfaceFrameCompletedTitleTexture =
    CreateFrame("Frame", "IronManInterfaceFrameCompletedTitleTexture", IronManInterfaceFrameCompleted)
IronManInterfaceFrameCompletedTitleTexture:SetSize(
    NumberList.interfaceTextureBoxSize,
    NumberList.interfaceTextureBoxSize
)
IronManInterfaceFrameCompletedTitleTexture:SetPoint("TOP", 0, NumberList.interfaceTextureGap)
IronManInterfaceFrameCompletedTitleTexture.Texture =
    IronManInterfaceFrameCompletedTitleTexture:CreateTexture(nil, "BACKGROUND")
IronManInterfaceFrameCompletedTitleTexture.Texture:SetPoint("CENTER")
IronManInterfaceFrameCompletedTitleTexture.Texture:SetScale(NumberList.interfaceTextureScale)
IronManInterfaceFrameCompletedTitleTexture.Texture:SetTexture(ImageList.IMBanner)

-- ===========================         scroll frames          ===========================

local IronManInterfaceScrollParent =
    CreateFrame("Frame", "IronManInterfaceScrollParent", IronManInterfaceFrameCompleted)
IronManInterfaceScrollParent:SetSize(NumberList.scrollParentWidth, NumberList.scrollParentHeight)
IronManInterfaceScrollParent:SetPoint(
    "TOP",
    IronManInterfaceFrameCompleted,
    "TOP",
    NumberList.scrollParentGapX,
    NumberList.scrollParentTopGap
)

local IronManInterfaceScrollFrame =
    CreateFrame("ScrollFrame", "IronManInterfaceScrollFrame", IronManInterfaceScrollParent, "FauxScrollFrameTemplate")
IronManInterfaceScrollFrame:SetPoint("TOPLEFT", 0, -8)
IronManInterfaceScrollFrame:SetPoint("BOTTOMRIGHT", -30, 8)
IronManInterfaceScrollFrame:SetScript(
    "OnVerticalScroll",
    function(self, offset)
        FauxScrollFrame_OnVerticalScroll(self, offset, NumberList.scrollFrameRowHeight, updateAchievementList)
    end
)
IronManInterfaceScrollFrame:SetScript(
    "OnShow",
    function(self, offset)
        updateAchievementList()
    end
)

IronManInterfaceScrollFrame.frames = {}
for index = 1, NumberList.scrollFrameNumberOfFrames do
    IronManInterfaceScrollFrame.frames[index] =
        CreateFrame("Frame", "IMISF" .. index, IronManInterfaceScrollParent, "HelpFrameContainerFrameTemplate")
    local frame = IronManInterfaceScrollFrame.frames[index]
    frame:SetSize(NumberList.scrollFrameRowWidth, NumberList.scrollFrameRowHeight)
    frame:SetPoint("TOPLEFT", 5, -(index - 1) * NumberList.scrollFrameRowHeight - 5)
    frame:Hide()

    frame.texture = frame:CreateTexture("frametexture" .. index)
    frame.texture:SetScale(NumberList.scrollAchievementTextureScale)
    frame.texture:SetPoint("LEFT")

    frame.titleTextBox = CreateFrame("Frame", "frametitleTextBox" .. index, frame)
    frame.titleTextBox:SetSize(NumberList.scrollAchievementTextBoxWidth, NumberList.scrollAchievementTextBoxHeight)
    frame.titleTextBox:SetPoint("TOPRIGHT")

    frame.titleTextBox.text = frame.titleTextBox:CreateFontString("frametitleTextBoxtext" .. index)
    frame.titleTextBox.text:SetAllPoints(frame.titleTextBox)
    frame.titleTextBox.text:SetFontObject(FontList.rulesFont)

    frame.frameNameTextBox = CreateFrame("Frame", "frameNameTextBox" .. index, frame)
    frame.frameNameTextBox:SetSize(NumberList.scrollAchievementTextBoxWidth, NumberList.scrollAchievementTextBoxHeight)
    frame.frameNameTextBox:SetPoint("RIGHT")

    frame.frameNameTextBox.text = frame.frameNameTextBox:CreateFontString("frameNameTextBoxtext" .. index)
    frame.frameNameTextBox.text:SetAllPoints(frame.frameNameTextBox)
    frame.frameNameTextBox.text:SetFontObject(FontList.rulesFont)

    frame.frameDateTextBox = CreateFrame("Frame", "frameDateTextBox" .. index, frame)
    frame.frameDateTextBox:SetSize(NumberList.scrollAchievementTextBoxWidth, NumberList.scrollAchievementTextBoxHeight)
    frame.frameDateTextBox:SetPoint("BOTTOMRIGHT")

    frame.frameDateTextBox.text = frame.frameDateTextBox:CreateFontString("frameDateTextBoxtext" .. index)
    frame.frameDateTextBox.text:SetAllPoints(frame.frameDateTextBox)
    frame.frameDateTextBox.text:SetFontObject(FontList.rulesFont)
end
